function data_prime=griddata_mesh_to_mesh(index,x,y,data,x_prime,y_prime,varargin);
%GRIDDATA_MESH_TO_MESH - interpolates data from a  2d mesh to a 2d mesh
%
%   griddata_mesh_to_mesh: interpolate data (from mesh with triangulation index,x,y)
%   into mesh with triangulation (x_prime,y_prime).
%
%   Usage:
%      data_prime=griddata_mesh_to_mesh(index,x,y,data,x_prime,y_prime,varargin)

	global client_server_mode IMDataCounter uset

	%restrict mesh_prime grids to the ones which are inside the mesh hull 
	in=inhull([x_prime y_prime],[x y]);
	pos_hull=find(in);

	%keep only mesh_prime grids inside the hull
	x_prime_hull=x_prime(pos_hull);
	y_prime_hull=y_prime(pos_hull);

	%call griddata_mesh_to_mesh_hulled on this new sub mesh from mesh 2
	if strcmpi(client_server_mode,'yes'),
		r_data_prime_hull=GriddataMeshToMeshHulled_client(index,x,y,data,x_prime_hull,y_prime_hull);
		data_prime_hull=IMdb('select matrix from r_data_prime_hull');
		IMdb('drop r_data_prime_hull');
	else
		data_prime_hull=griddata_mesh_to_mesh_hulled(index,x,y,data,x_prime_hull,y_prime_hull,varargin);
	end

	%return output
	data_prime=zeros(length(x_prime),1); data_prime(:)=NaN;
	data_prime(pos_hull)=data_prime_hull;
end

function data_prime=griddata_mesh_to_mesh_hulled(index,x,y,data,x_prime,y_prime,varargin);

	%Analyse data: is it given per node or per element?
	if isempty(varargin{1})
		element_data=(length(data)==size(index,1));%data is given for a given element
		grid_data=(length(data)==length(x));       %data is given for a given grid
		if element_data==0 & grid_data==0, error('griddata_mesh_to_mesh_hulled error message: unknown size of data, specify element or node'), end
	else
		if strcmpi(varargin{1},'node')
			element_data=0;
			grid_data=1;
		elseif strcmpi(varargin{1},'element')
			element_data=1;
			grid_data=0;
		else
			error('griddata_mesh_to_mesh_hulled error message: wrong input argument. Must be ''node'' or ''element''')
		end
	end

	%Some parameters and initialization of data_prime
	nel=length(index);
	data_prime=zeros(length(x_prime),1); data_prime(:)=NaN;

	%compute some quantities that will speed up the process
	x3x1=x(index(:,1))-x(index(:,3));
	y3y1=y(index(:,1))-y(index(:,3));
	x3x2=x(index(:,2))-x(index(:,3));
	y3y2=y(index(:,2))-y(index(:,3));
	x3=x(index(:,3));
	y3=y(index(:,3));

	%triangles jacobian determinant (2*area)
	delta=x(index(:,2)).*y(index(:,3))-y(index(:,2)).*x(index(:,3))-x(index(:,1)).*y(index(:,3))+y(index(:,1)).*x(index(:,3))+x(index(:,1)).*y(index(:,2))-y(index(:,1)).*x(index(:,2));

	if nel>100,
		fprintf('%s','      interpolation progress:   0.00 %');
	end

	%Get area coordinates:
	for n=1:nel,
		if mod(n,100)==0,
			fprintf('\b\b\b\b\b\b\b')
			fprintf('%5.2f%s',n/nel*100,' %');
		end
		%first area coordinate
		area_1=(y3y2(n)*(x_prime-x3(n))-x3x2(n)*(y_prime-y3(n)))/delta(n);
		%second area coordinate
		area_2=(x3x1(n)*(y_prime-y3(n))-y3y1(n)*(x_prime-x3(n)))/delta(n);
		%third area coordinate
		area_3=1-area_1-area_2;
		%get grids in mesh 2 that have all area coordinates positive (meaning they belong to mesh 1 triangle n:
		pos=find((area_1>=0) & (area_2>=0) & (area_3>=0));
		if ~isempty(pos),
			if grid_data
				%mesh 2 grids in pos belong to mesh 1 element n. interpolate value linearly.
				data_prime(pos)=area_1(pos)*data(index(n,1))+area_2(pos)*data(index(n,2))+area_3(pos)*data(index(n,3));
			elseif element_data
				data_prime(pos)=data(n);
			end
		end
	end
	if nel>100,
		fprintf('\b\b\b\b\b\b\b\b')
		fprintf('%4.2f%s\n',100,' %');
	end
end
